@@ -31,6 +31,8 @@ class EvernoteClientSync(EvernoteClient):  # noqa: WPS214
         self.max_chunk_results = max_chunk_results
 
     def get_note(self, note_guid: str) -> Note:
+        logger.debug(f"Downloading note [{note_guid}]")
+
         note = self.note_store.getNote(
             note_guid, True, True, True, True  # noqa: WPS425
         )
@@ -38,6 +40,8 @@ class EvernoteClientSync(EvernoteClient):  # noqa: WPS214
         if note.tagGuids:
             note.tagNames = [self.tags[t] for t in note.tagGuids]
 
+        logger.debug(f"Finished downloading note [{note.guid}]")
+
         return note
 
     def iter_sync_chunks(self, after_usn: int) -> Iterator[SyncChunk]:
