@@ -185,7 +185,7 @@ class NoteBookStorage(SqliteStorage):  # noqa: WPS214
     ) -> None:
         if logger.getEffectiveLevel() == logging.DEBUG:  # pragma: no cover
             logger.debug(
-                f"Adding/updating linked notebook {l_notebook.shareName}"
+                f"Adding/updating linked notebook '{l_notebook.shareName}'"
                 f" [{l_notebook.guid}] -> [{notebook.guid}]"
             )
 
@@ -280,6 +280,8 @@ class NoteStorage(SqliteStorage):  # noqa: WPS214
                 ),
             )
 
+        logger.debug(f"Added note [{note.guid}]")
+
     def iter_notes(self, notebook_guid: str) -> Iterator[Note]:
         with self.db as con:
             cur = con.execute(
